<?php

namespace FiloBlu\Refilo\Model\Exporter;

use FiloBlu\Core\Model\Configuration;
use FiloBlu\Refilo\Helper\PageBuilderConverter;
use FiloBlu\Refilo\Model\Adapter\Mongo;
use Magento\Cms\Model\ResourceModel\Block\CollectionFactory as BlockCollectionFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class CmsBlock
 * @package FiloBlu\Refilo\Model\Exporter
 */
class CmsBlock extends AbstractCmsExporter
{
    /**
     * @var BlockCollectionFactory
     */
    protected $blockCollectionFactory;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;
    /**
     * @var PageBuilderConverter
     */
    private $pageBuilderConverter;


    /**
     * Menu constructor.
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param StoreManagerInterface $storeManager
     * @param Configuration $coreConfiguration
     * @param BlockCollectionFactory $blockCollectionFactory
     */
    public function __construct(
        ResourceConnection $resourceConnection,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        StoreManagerInterface $storeManager,
        Configuration $coreConfiguration,
        BlockCollectionFactory $blockCollectionFactory,
        PageBuilderConverter $pageBuilderConverter
    )
    {
        parent::__construct($mongo, $coreConfiguration ,$scopeConfig, $storeManager);
        $this->resourceConnection = $resourceConnection;
        $this->blockCollectionFactory = $blockCollectionFactory;
        $this->pageBuilderConverter = $pageBuilderConverter;
    }


    public function getMetaCode(): string
    {
        return 'block';
    }

    public function execute(OutputInterface $output = null)
    {
        /** @var Store $store */
        foreach ($this->getStoreManager()->getStores() as $store) {

            $blocks = $this->blockCollectionFactory->create()
                ->addStoreFilter($store)
                ->addFilter('is_active', 1);

            $bulk = [];
            foreach ($blocks as $item) {
                $bulk[] = [
                    '_id'                      => 'block_' . $item->getId() . '_' . $store->getData('code'),
                    'metacode'                 => 'block',
                    'store'                    => $store->getData('code'),
                    'content_id'               => (int)$item->getId(),
                    'url_key'                  => $item->getIdentifier(),
                    'idx'                      => $item->getIdentifier(),
                    'title'                    => $item->getTitle(),
                    'type'                     => 'static_block',
                    'status'                   => $item->isActive(),
                    'content'                  => $this->pageBuilderConverter->convert($item->getContent()),
                    'display_entity'           => $item->getDisplayOn(),
                    'display_selection_type'   => $item->getSelectionType(),
                    'display_selection_target' => $this->parseTargetSelection($item->getSelectionTarget()),
                    'display_container'        => $item->getRefiloLayoutContainer(),
                    'display_sort_order'       => (int)$item->getRefiloLayoutOrder(),
                    'listing_display_page'     => (int)$item->getListingDisplayPage(),
                    'listing_display_position' => (int)$item->getListingDisplayPosition(),
                    'listing_columns_spacing'  => (int)$item->getListingColumnsSpacing(),
                    'listing_row_spacing'      => (int)$item->getListingRowSpacing(),
                    'customer_group'           => $item->getCustomerGroup() !== null ? array_map('intval', explode(',', $item->getCustomerGroup())) : null
                ];
            }

            try {
                $this->getConnection()->update(
                    '_id',
                    $bulk,
                    'contents_' . $store->getCode(),
                    true,
                    $this->getDatabaseByWebsiteCode($store->getWebsite()->getCode())
                );
            } catch (\Exception $e) {
                echo $e->getMessage();
            }
        }
    }

    /**
     * @param $value
     * @return false|string[]
     */
    protected function parseTargetSelection($value)
    {
        $strippedValue = preg_replace('/\s/', '', $value ??'');
        return explode(',', $strippedValue);
    }

    public function getCollectionPrefix(): string
    {
        return 'contents_';
    }
}
